package cn.iocoder.yudao.module.shipment.service.boxApproval;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.bpm.api.BpmCreateServiceFactory;
import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.enums.WorkFlowEmus;
import cn.iocoder.yudao.module.bpm.resp.TaskResp;
import cn.iocoder.yudao.module.ecw.service.internalMessage.InternalMessageService;
import cn.iocoder.yudao.module.ecw.vo.internalMessage.InternalMessageCreateReqVO;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.order.zTest;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderSplitItem.OrderSplitItemDO;
import cn.iocoder.yudao.module.order.dal.mysql.order.ZTestMapper2;
import cn.iocoder.yudao.module.order.enums.*;
import cn.iocoder.yudao.module.order.service.targetLog.TargetLogService;
import cn.iocoder.yudao.module.order.vo.order.CostVO;
import cn.iocoder.yudao.module.order.vo.order.WarehouseInInfoVO;
import cn.iocoder.yudao.module.order.vo.orderSplit.OrderSplitApplyDetailDto;
import cn.iocoder.yudao.module.order.vo.orderSplit.OrderSplitBackVO;
import cn.iocoder.yudao.module.order.vo.orderSplitItem.OrderSplitItemBackVO;
import cn.iocoder.yudao.module.shipment.convert.BoxApprovalConvert;
import cn.iocoder.yudao.module.shipment.convert.BoxCabinetConvert;
import cn.iocoder.yudao.module.shipment.convert.BoxCabinetUnloadConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.*;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxOrderExit.BoxOrderExitDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxApprovalMapper;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxBookSeaMapper;
import cn.iocoder.yudao.module.shipment.dto.CustomsExitType;
import cn.iocoder.yudao.module.shipment.enums.*;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.service.boxOrderExit.BoxOrderExitService;
import cn.iocoder.yudao.module.shipment.vo.box.BoxBaseBackVO;
import cn.iocoder.yudao.module.shipment.vo.box.BoxOrderMarkUpVO;
import cn.iocoder.yudao.module.shipment.vo.boxApproval.BoxLoadDetailVO;
import cn.iocoder.yudao.module.shipment.vo.boxApproval.BoxApprovalCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxApproval.BoxApprovalQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxApproval.BoxApprovalUpdateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxLoadOrderVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadSection.BoxLoadDetailBackVO;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.module.order.enums.TargetLogEnum.LOADING;
import static cn.iocoder.yudao.module.order.enums.TargetLogEnum.WITHDRAWAL;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.*;

/**
 * 出货审核 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxApprovalServiceImpl extends AbstractService<BoxApprovalMapper, BoxApprovalDO> implements BoxApprovalService {

    @Resource
    private ZTestMapper2 zTestMapper2;

    @Resource
    private BoxApprovalMapper boxApprovalMapper;
    @Resource
    BoxOpLogService boxOpLogService;

    @Resource
    TargetLogService targetLogService;
    @Resource
    @Lazy
    BoxService boxService;
    @Resource
    private BoxBookSeaMapper boxBookSeaMapper;
    @Resource
    BoxOrderExitService boxOrderExitService;
    @Resource
    BpmCreateServiceFactory bpmCreateServiceFactory;

    @Resource
    private InternalMessageService internalMessageService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxApproval(BoxApprovalCreateReqVO createReqVO) {
        int approvalType = createReqVO.getApprovalType();
        // 插入
//        checkApproval(createReqVO.getShipmentId(), approvalType);
        //同时只能有一个在审核中
        checkApproval(createReqVO.getShipmentId(), null);
        //校验订单是否有在审核中的
        boxService.checkOrdersApprovaling(createReqVO.getShipmentId());

        BoxApprovalDO boxApproval = BoxApprovalConvert.INSTANCE.convert(createReqVO);
        boxApproval.setApprovalStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        boxApprovalMapper.insert(boxApproval);

        Long shipmentId = createReqVO.getShipmentId();
        //修改预装状态
        BoxDO boxDO = boxService.getBox(shipmentId);

        String flowKey = "";

        //----------------------------------------预装---------------------------------------
        if (createReqVO.getApprovalType() ==
                BoxApprovalTypeEnum.PRELOAD.getType()) {
            //检查是否有预装订单
            boxService.checkPreInstallNum(shipmentId);

            //预装
            flowKey = WorkFlowEmus.SHIPMENT_PREASSEMBLE.getKey();
            PrStatusEnum prStatusEnum = PrStatusEnum.PRELOAD_APPROVAL_ING;
            boxDO.setPrStatus(prStatusEnum.getPrStatus());
            boxService.updateById(boxDO);

            //添加操作日志
            boxOpLogService.createBoxOpLog(boxDO.getId(), prStatusEnum.getPrStatus(), prStatusEnum.getStatusNameZh(), prStatusEnum.getStatusNameEn(), boxApproval.getId());

            OrderApprovalTypeResultEnum pre_install_processing = OrderApprovalTypeResultEnum.pre_install_processing;
            //修改为预装审核中、预装时需要将入仓子状态置0
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, 0,
                    BoxStatusEnum.PREINSTALL_EXAMINE.getStatus(), pre_install_processing.getType(), pre_install_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));

        } else if (createReqVO.getApprovalType() ==
                BoxApprovalTypeEnum.SEALING_CABINET.getType()) {
            //首先校验出货是否已经预装审核成功
            Integer prStatus = boxDO.getPrStatus();
            if(prStatus != PrStatusEnum.PRELOAD_APPROVAL_SUCCESS.getPrStatus()) {
                throw new ServiceException(500, "出货没有预装审核成功");
            }

            //封柜
            flowKey = WorkFlowEmus.SHIPMENT_CLOSE_CONTAINER.getKey();
            //装柜，校验已装箱数量是否符合
            boxService.checkInstallNum(shipmentId);
            LdStatusEnum ldStatusEnum = LdStatusEnum.CABINET_EXAMINE_ING;
            boxDO.setLdStatus(ldStatusEnum.getLdStatus());
            boxService.updateById(boxDO);

            OrderApprovalTypeResultEnum closure_processing = OrderApprovalTypeResultEnum.closure_processing;
            //修改为封柜审核中
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.INSTALLED_CABINET.getValue(), null,
                    BoxStatusEnum.SEAL_EXAMINE.getStatus(), closure_processing.getType(), closure_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));

            //封柜申请日志
            boxOpLogService.createBoxOpLog(boxDO.getId(), ldStatusEnum.getLdStatus(), ldStatusEnum.getStatusNameZh(), ldStatusEnum.getStatusNameEn(), boxApproval.getId());
        }
        //------------------------------------------卸柜--------------------------------------
        else if (createReqVO.getApprovalType() == BoxApprovalTypeEnum.UNLOAD_CABINET.getType()) {
            //卸柜
            flowKey = WorkFlowEmus.SHIPMENT_UNLOAD_CONTAINER.getKey();
            //卸柜，校验卸柜数量是否符合
            boxService.checkUnLoadNum(shipmentId);
            ULStatusEnum ulStatusEnum = ULStatusEnum.UNLOADING_EXAMINE_ING;
            boxDO.setUlStatus(ulStatusEnum.getUlStatus());
            boxService.updateById(boxDO);

            String transportType = boxDO.getTransportType();
            Integer orderStatus = OrderStatusEnum.CLEARED.getValue();
            if (transportType.equals(TransportTypeEnum.SEA_AIR.getType())) {
                orderStatus = OrderStatusEnum.TWP_WAY_ARRIVAL.getValue();
            }

            OrderApprovalTypeResultEnum unloading_cabinet_processing = OrderApprovalTypeResultEnum.unloading_cabinet_processing;
            //修改为卸柜审核中
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, orderStatus, null,
                    BoxStatusEnum.UNLOAD_EXAMINE.getStatus(), unloading_cabinet_processing.getType(),
                    unloading_cabinet_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));

            //卸柜审核日志
            boxOpLogService.createBoxOpLog(shipmentId, ulStatusEnum.getUlStatus(), ulStatusEnum.getStatusNameZh(), ulStatusEnum.getStatusNameEn(), boxApproval.getId());
        }
        //-------------------------------------------预装反审----------------------------------------
        else if (createReqVO.getApprovalType() ==
                BoxApprovalTypeEnum.PRELOAD_UMPIRE.getType()) {
            //预装反审
            flowKey = WorkFlowEmus.SHIPMENT_PREASSEMBLE_NO.getKey();

            //添加操作日志
            boxOpLogService.createBoxOpLog(boxDO.getId(), PrStatusEnum.PRELOAD_BACK_APPROVAL_ING.getPrStatus(),
                    PrStatusEnum.PRELOAD_BACK_APPROVAL_ING.getStatusNameZh(), PrStatusEnum.PRELOAD_BACK_APPROVAL_ING.getStatusNameEn(), boxApproval.getId());

            OrderApprovalTypeResultEnum pre_install_back_processing = OrderApprovalTypeResultEnum.pre_install_back_processing;
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                    null, pre_install_back_processing.getType(), pre_install_back_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));

        }
        //------------------------------------------封柜反审-------------------------------------------
        else if (createReqVO.getApprovalType() == BoxApprovalTypeEnum.SEALING_CABINET_UMPIRE.getType()) {
            //封柜反审
            flowKey = WorkFlowEmus.SHIPMENT_CLOSE_CONTAINER_NO.getKey();

            //封柜反审日志
            boxOpLogService.createBoxOpLog(boxDO.getId(), LdStatusEnum.CABINET_BACK_EXAMINE_ING.getLdStatus(),
                    LdStatusEnum.CABINET_BACK_EXAMINE_ING.getStatusNameZh(), LdStatusEnum.CABINET_BACK_EXAMINE_ING.getStatusNameEn(), boxApproval.getId());

            OrderApprovalTypeResultEnum closure_back_processing = OrderApprovalTypeResultEnum.closure_back_processing;
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                    null, closure_back_processing.getType(), closure_back_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));

        }
        //-------------------------------------------报关全退----------------------------------------
        else if (createReqVO.getApprovalType() == BoxApprovalTypeEnum.CUSTOMS_FULL_REFUND.getType()) {
            checkCustomsApproval(boxDO);
            boxDO.setDcStatus(DcStatueEnum.CUSTOMING.getDcStatus());
            boxService.updateById(boxDO);

            //报关全退
            flowKey = WorkFlowEmus.SHIPMENT_CUSTOMS_EXIT_ALL.getKey();

            //报关全退日志
            int opStatus = Integer.parseInt(DcStatueEnum.CUSTOMING.getDcStatus() + "1");
            boxOpLogService.createBoxOpLog(boxDO.getId(), opStatus, "报关全部退场审核中", "Full return of customs declaration under review", boxApproval.getId());

            OrderApprovalTypeResultEnum customs_exit_processing = OrderApprovalTypeResultEnum.customs_exit_processing;
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                    null, customs_exit_processing.getType(), customs_exit_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));

        }
        //------------------------------------------------报关部分退-------------------------------------
        else if (createReqVO.getApprovalType() == BoxApprovalTypeEnum.CUSTOMS_PART_REFUND.getType()) {
            checkCustomsApproval(boxDO);
            boxDO.setDcStatus(DcStatueEnum.CUSTOMING.getDcStatus());
            boxService.updateById(boxDO);

            //报关部分退
            flowKey = WorkFlowEmus.SHIPMENT_CUSTOMS_EXIT_PART.getKey();

            //报关部分退日志
            int opStatus = Integer.parseInt(DcStatueEnum.CUSTOMING.getDcStatus() + "2");
            boxOpLogService.createBoxOpLog(boxDO.getId(), opStatus, "报关部分退场审核中", "The customs clearance part is undergoing exit review", boxApproval.getId());


            OrderApprovalTypeResultEnum customs_exit_processing = OrderApprovalTypeResultEnum.customs_exit_processing;
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                    null, customs_exit_processing.getType(), customs_exit_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));

        }
        //---------------------------------------------卸柜反审-------------------------------------
        else if (createReqVO.getApprovalType() == BoxApprovalTypeEnum.UNLOAD_UMPIRE.getType()) {
            //卸柜反审
            flowKey = WorkFlowEmus.SHIPMENT_UNLOAD_CONTAINER_NO.getKey();

            //卸柜审核日志
            boxOpLogService.createBoxOpLog(shipmentId, ULStatusEnum.UNLOADING_BACK_EXAMINE_ING.getUlStatus(),
                    ULStatusEnum.UNLOADING_BACK_EXAMINE_ING.getStatusNameZh(), ULStatusEnum.UNLOADING_BACK_EXAMINE_ING.getStatusNameEn(), boxApproval.getId());

            OrderApprovalTypeResultEnum unloading_cabinet_back_processing = OrderApprovalTypeResultEnum.unloading_cabinet_back_processing;
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                    null, unloading_cabinet_back_processing.getType(), unloading_cabinet_back_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));

        }
        //--------------------------------------出货拆单-----------------------------------
        else if (createReqVO.getApprovalType() == BoxApprovalTypeEnum.SPLIT_ORDER.getType()) { //出货拆单申请
            flowKey = WorkFlowEmus.SHIPMENT_SPLIT_ORDER.getKey();

            Long orderId = createReqVO.getOrderId();

            //申请信息组装
            OrderSplitApplyDetailDto orderSplitApplyDetailDto = new OrderSplitApplyDetailDto();
            orderSplitApplyDetailDto.setOrderId(orderId);
            // 拆单申请detail
            List<OrderSplitBackVO> orderSplitBackVOList = boxService.listByOrderId(orderId);
            if (CollectionUtil.isEmpty(orderSplitBackVOList)) {
                throw exception(ORDER_SPLIT_DATA_NOT_EXITS);
            }
            if (orderSplitBackVOList.size() < 2){
                throw exception(ORDER_SPLIT_AT_LEAST_TWO);
            }
            // 校验拆单申请的数据是否合法(注：拆出的订单项值不能为空)
            for (OrderSplitBackVO orderSplitBackVO : orderSplitBackVOList) {
                if (CollectionUtil.isEmpty(orderSplitBackVO.getOrderSplitItemBackVOList())) {
                    // 拆分的订单项为空抛出异常
                    throw exception(SPLIT_ORDER_ITEM_NOT_NULL, orderSplitBackVO.getOrderNo());
                }
            }
            OrderDO orderDO = boxService.getOrder(orderId);
            if (OrderStatusEnum.DRAFT.getValue().equals(orderDO.getStatus())) {
                throw exception(ORDER_NOT_EXISTS);
            }
            // 货值需要匹配整个订单的
            BigDecimal totalWorth = BigDecimal.ZERO;
            if (StringUtils.isNotBlank(orderDO.getCost())) {
                CostVO costVO = JSONObject.parseObject(orderDO.getCost(), CostVO.class);
                totalWorth = Objects.nonNull(costVO.getTotalWorth()) ? costVO.getTotalWorth() : BigDecimal.ZERO;
            }
            List<OrderItemDO> orderItems = boxService.getOrderItemsByOrderId(orderId);
            List<OrderSplitItemDO> orderAllSplitItemDOList = new ArrayList<>();
            for (OrderItemDO orderItemDO : orderItems) {
                WarehouseInInfoVO warehouseInInfoVO = orderItemDO.getWarehouseInInfoVO();
                if (Objects.nonNull(warehouseInInfoVO) && warehouseInInfoVO.getCartonsNum() > 0) {
                    // 校验所有拆单项与原单的强一致
                    List<OrderSplitItemDO> orderSplitItemDOList = boxService.splitItemListByOrderId(orderItemDO.getOrderItemId());
                    if (CollectionUtil.isEmpty(orderSplitItemDOList)){
                        throw exception(SPLIT_ORDER_ITEM_PROD_NOT_NULL, I18nMessage.getLang() == 0 ? orderItemDO.getProdTitleZh() : orderItemDO.getProdTitleEn());
                    }
                    boxService.splitConsistencyCheck(orderSplitItemDOList, warehouseInInfoVO.getCartonsNum(), warehouseInInfoVO.getQuantityAll(),
                            warehouseInInfoVO.getWeight(), warehouseInInfoVO.getVolume(), orderItemDO.getChargeWeight(),
                            orderItemDO.getChargeVolume(), orderItemDO.getChargeQuantity());
                    orderAllSplitItemDOList.addAll(orderSplitItemDOList);
                }
            }
            // 校验所有拆单项货值与原单总货值的强一致
            boxService.splitWorthConsistencyCheck(orderAllSplitItemDOList, totalWorth);
            orderSplitApplyDetailDto.setOrderSplitBackVOList(orderSplitBackVOList);

            boxApproval.setDetails(JSON.toJSONString(orderSplitApplyDetailDto));

            OrderApprovalTypeResultEnum order_split_processing = OrderApprovalTypeResultEnum.order_split_processing;
            //更新订单状态为拆单申请中
            boxService.updateStatus(orderId, null, null, null,
                    null, null, order_split_processing.getType(), order_split_processing.getDesc());

            boxService.addOrderLog(Collections.singletonList(orderId), OrderShipmentLog.CABINET_SPLIT_REQUEST, "");
        }
        //------------------------------------------部分退场拆单、 删单退场拆单审核-------------------------------------
        else if (createReqVO.getApprovalType() == BoxApprovalTypeEnum.PART_REFUND_SPLIT_ORDER.getType() || createReqVO.getApprovalType() == BoxApprovalTypeEnum.DELETE_EXIT_SPLIT_ORDER.getType()) {  //出货拆单申请
            //部分拆单退场、删单退场拆单
            flowKey = createReqVO.getApprovalType() == BoxApprovalTypeEnum.PART_REFUND_SPLIT_ORDER.getType() ? WorkFlowEmus.SHIPMENT_EXIT_PART_SPILT.getKey() : WorkFlowEmus.SHIPMENT_CUSTOMS_DECLARE_REMOVE_SPLIT.getKey();

            Long orderId = createReqVO.getOrderId();

            //申请信息组装
            OrderSplitApplyDetailDto orderSplitApplyDetailDto = new OrderSplitApplyDetailDto();
            orderSplitApplyDetailDto.setOrderId(orderId);
            // 拆单申请detail
            List<OrderSplitBackVO> orderSplitBackVOList = boxService.listByOrderId(orderId);
            if (CollectionUtil.isEmpty(orderSplitBackVOList)) {
                throw exception(ORDER_SPLIT_DATA_NOT_EXITS);
            }
            if (orderSplitBackVOList.size() < 2){
                throw exception(ORDER_SPLIT_AT_LEAST_TWO);
            }
            // 校验拆单申请的数据是否合法(注：拆出的订单项值不能为空)
            for (OrderSplitBackVO orderSplitBackVO : orderSplitBackVOList) {
                if (CollectionUtil.isEmpty(orderSplitBackVO.getOrderSplitItemBackVOList())) {
                    // 拆分的订单项为空抛出异常
                    throw exception(SPLIT_ORDER_ITEM_NOT_NULL, orderSplitBackVO.getOrderNo());
                }
            }
            OrderDO orderDO = boxService.getOrder(orderId);
            if (OrderStatusEnum.DRAFT.getValue().equals(orderDO.getStatus())) {
                throw exception(ORDER_NOT_EXISTS);
            }
            // 货值需要匹配整个订单的
            BigDecimal totalWorth = BigDecimal.ZERO;
            if (StringUtils.isNotBlank(orderDO.getCost())) {
                CostVO costVO = JSONObject.parseObject(orderDO.getCost(), CostVO.class);
                totalWorth = Objects.nonNull(costVO.getTotalWorth()) ? costVO.getTotalWorth() : BigDecimal.ZERO;
            }
            List<OrderItemDO> orderItems = boxService.getOrderItemsByOrderId(orderId);
            List<OrderSplitItemDO> orderAllSplitItemDOList = new ArrayList<>();
            for (OrderItemDO orderItemDO : orderItems) {
                WarehouseInInfoVO warehouseInInfoVO = orderItemDO.getWarehouseInInfoVO();
                if (Objects.nonNull(warehouseInInfoVO) && warehouseInInfoVO.getCartonsNum() > 0) {
                    // 校验所有拆单项与原单的强一致
                    List<OrderSplitItemDO> orderSplitItemDOList = boxService.splitItemListByOrderId(orderItemDO.getOrderItemId());
                    if (CollectionUtil.isEmpty(orderSplitItemDOList)){
                        throw exception(SPLIT_ORDER_ITEM_PROD_NOT_NULL, I18nMessage.getLang() == 0 ? orderItemDO.getProdTitleZh() : orderItemDO.getProdTitleEn());
                    }
                    boxService.splitConsistencyCheck(orderSplitItemDOList, warehouseInInfoVO.getCartonsNum(), warehouseInInfoVO.getQuantityAll(),
                            warehouseInInfoVO.getWeight(), warehouseInInfoVO.getVolume(), orderItemDO.getChargeWeight(),
                            orderItemDO.getChargeVolume(), orderItemDO.getChargeQuantity());
                    orderAllSplitItemDOList.addAll(orderSplitItemDOList);
                }
            }
            // 校验所有拆单项货值与原单总货值的强一致
            boxService.splitWorthConsistencyCheck(orderAllSplitItemDOList, totalWorth);
            int refundNum = orderSplitBackVOList.stream().filter(OrderSplitBackVO::getIsMaster).findFirst().get().getRefundNum();
            // 部分退场拆单需要校验退场箱数是否一致
            for (OrderSplitBackVO split : orderSplitBackVOList) {
                if (!split.getIsMaster()) {
                    int num = split.getOrderSplitItemBackVOList().stream().mapToInt(OrderSplitItemBackVO::getNum).sum();
                    refundNum = refundNum - num;
                }
            }
            if (refundNum != 0) {
                throw exception(PART_REFUND_ORDER_NUM_NOT_CORRECT);
            }
            orderSplitApplyDetailDto.setOrderSplitBackVOList(orderSplitBackVOList);

            boxApproval.setDetails(JSON.toJSONString(orderSplitApplyDetailDto));

            OrderApprovalTypeResultEnum order_split_processing = OrderApprovalTypeResultEnum.order_split_processing;
            // 部分退场待拆单订单标记为拆单中
            boxOrderExitService.update(new LambdaUpdateWrapper<BoxOrderExitDO>()
                    .set(BoxOrderExitDO::getSplitStatus, 2)
                    .eq(BoxOrderExitDO::getOrderId, orderSplitApplyDetailDto.getOrderId())
                    .eq(BoxOrderExitDO::getShipmentId, shipmentId));
            //更新订单状态为拆单申请中
            boxService.updateStatus(orderId, null, null, null,
                    null, null, order_split_processing.getType(), order_split_processing.getDesc());
            OrderShipmentLog orderShipmentLog = createReqVO.getApprovalType() == BoxApprovalTypeEnum.PART_REFUND_SPLIT_ORDER.getType() ?
                    OrderShipmentLog.PART_REFUND_SPLIT_REQUEST:OrderShipmentLog.DELETE_ORDER_REFUND_SPLIT_REQUEST;
            boxService.addOrderLog(Collections.singletonList(orderId), orderShipmentLog, "");
        }
        //-----------------------------------------批量加价-------------------------------------------
        else if(createReqVO.getApprovalType() ==
                BoxApprovalTypeEnum.BATCH_ADD_PRICE.getType()) {
            //批量加价
            flowKey = WorkFlowEmus.SHIPMENT_BATCH_ADD_PRICE.getKey();
            String details = createReqVO.getDetails();
            BoxOrderMarkUpVO boxOrderMarkUpVO =
                    JSON.parseObject(details, BoxOrderMarkUpVO.class);
            boxService.checkOrderPayStatus(boxOrderMarkUpVO.getOrderIds());
        }
        //-----------------------------------------删单退场-------------------------------------------
        else if(createReqVO.getApprovalType() == BoxApprovalTypeEnum.DELETE_EXIT.getType()) {
            checkCustomsApproval(boxDO);

            flowKey = WorkFlowEmus.SHIPMENT_CUSTOMS_DECLARE_REMOVE.getKey();

            int opStatus = Integer.parseInt(DcStatueEnum.CUSTOMING.getDcStatus() + "3");
            boxOpLogService.createBoxOpLog(boxDO.getId(), opStatus, "报关删单退场审核中", "The customs clearance part is undergoing exit review", boxApproval.getId());

            OrderApprovalTypeResultEnum customs_exit_processing = OrderApprovalTypeResultEnum.customs_exit_processing;
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                    null, customs_exit_processing.getType(), customs_exit_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));

            boxDO.setOverMachineExamineStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
            CustomsExitType customsExitType = JSON.parseObject(boxApproval.getDetails(), CustomsExitType.class);
            boxDO.setDeleteType(customsExitType.getDeleteExitType());
            boxDO.setOverMachineStatus(2);
            boxDO.setOverMachineAbnormalStatus(1);
            boxDO.setDcStatus(DcStatueEnum.CUSTOMING.getDcStatus());
            if(customsExitType.getDeleteExitType() == 1) {
                boxApproval.setApplyReason("退场时间: " + customsExitType.getDeleteExitTime());
            } else {
                boxApproval.setApplyReason("返回仓库");
            }
            boxService.updateById(boxDO);
        }
        //------------------------------------------到仓--------------------------------------
        else if(createReqVO.getApprovalType() == BoxApprovalTypeEnum.AIR_TO_WAREHOUSE.getType()) {
            flowKey = WorkFlowEmus.SHIPMENT_TO_WAREHOUSE.getKey();
            //到仓，校验到仓数量是否符合
            boxService.checkUnLoadNum(shipmentId);
            TWStatusEnum twStatusEnum = TWStatusEnum.TO_WAREHOUSE_EXAMINE_ING;
            boxDO.setToWarehouseStatus(twStatusEnum.getTwStatus());
            boxService.updateById(boxDO);

            OrderApprovalTypeResultEnum warehoused_processing = OrderApprovalTypeResultEnum.warehoused_processing;
            //修改为到仓审核中
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.CLEARED.getValue(), null,
                    BoxAirStatusEnum.TO_WAREHOUSE_EXAMINE.getStatus(), warehoused_processing.getType(),
                    warehoused_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));

            //到仓审核日志
            boxOpLogService.createBoxOpLog(shipmentId, twStatusEnum.getTwStatus(), twStatusEnum.getStatusNameZh(), twStatusEnum.getStatusNameEn(), boxApproval.getId());

        }
        //---------------------------------------到仓反审--------------------------------
        else if (createReqVO.getApprovalType() == BoxApprovalTypeEnum.AIR_TO_WAREHOUSE_BACK.getType()) {
            flowKey = WorkFlowEmus.SHIPMENT_TO_WAREHOUSE_No.getKey();

            //到仓反审日志
            boxOpLogService.createBoxOpLog(shipmentId, TWStatusEnum.TO_WAREHOUSE_BACK_EXAMINE_ING.getTwStatus(),
                    TWStatusEnum.TO_WAREHOUSE_BACK_EXAMINE_ING.getStatusNameZh(), TWStatusEnum.TO_WAREHOUSE_BACK_EXAMINE_ING.getStatusNameEn(), boxApproval.getId());

            OrderApprovalTypeResultEnum warehoused_back_processing = OrderApprovalTypeResultEnum.warehoused_back_processing;
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                    null, warehoused_back_processing.getType(), warehoused_back_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));

        }

        //------------------------------------分拣--------------------------------------
        else if (createReqVO.getApprovalType() == BoxApprovalTypeEnum.AIR_SORTING.getType()) {
            //检查是否有分拣订单
            boxService.checkPreInstallNum(shipmentId);

            //分拣
            flowKey = WorkFlowEmus.SHIPMENT_ORDER_SORTING.getKey();
            StStatusEnum stStatusEnum = StStatusEnum.SORTING_APPROVAL_ING;
            boxDO.setPrStatus(stStatusEnum.getStStatus());
            boxService.updateById(boxDO);

            //添加操作日志
            boxOpLogService.createBoxOpLog(boxDO.getId(), stStatusEnum.getStStatus(), stStatusEnum.getStatusNameZh(), stStatusEnum.getStatusNameEn(), boxApproval.getId());

            OrderApprovalTypeResultEnum sorting_processing = OrderApprovalTypeResultEnum.sorting_processing;
            //修改为分拣审核中、分拣时需要将入仓子状态置0 ? 空运的是否需要修改???
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, 0,
                    BoxStatusEnum.PREINSTALL_EXAMINE.getStatus(), sorting_processing.getType(), sorting_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));
        }

        //------------------------------------------分拣反审核-------------------------
        else if (createReqVO.getApprovalType() == BoxApprovalTypeEnum.AIR_SORTING_BACK.getType()) {
            flowKey = WorkFlowEmus.SHIPMENT_ORDER_SORTING_NO.getKey();

            //添加操作日志
            boxOpLogService.createBoxOpLog(boxDO.getId(), StStatusEnum.SORTING_BACK_APPROVAL_ING.getStStatus(),
                    StStatusEnum.SORTING_BACK_APPROVAL_ING.getStatusNameZh(), StStatusEnum.SORTING_BACK_APPROVAL_ING.getStatusNameEn(), boxApproval.getId());

            OrderApprovalTypeResultEnum sorting_back_processing = OrderApprovalTypeResultEnum.sorting_back_processing;
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                    null, sorting_back_processing.getType(), sorting_back_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));

        }

        //----------------------------------------------出货审核-----------------------------------------
        else if(createReqVO.getApprovalType() == BoxApprovalTypeEnum.AIR_SHIPMENT.getType()) {
            flowKey = WorkFlowEmus.SHIPMENT_AUDIT.getKey();

            //添加操作日志
            boxOpLogService.createBoxOpLog(boxDO.getId(), AirShipmentStatusEnum.SHIPMENT_APPROVAL_ING.getStatus(),
                    AirShipmentStatusEnum.SHIPMENT_APPROVAL_ING.getText(), AirShipmentStatusEnum.SHIPMENT_APPROVAL_ING.getTextEn(), boxApproval.getId());

            OrderApprovalTypeResultEnum shipment_processing = OrderApprovalTypeResultEnum.shipment_processing;
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                    null, shipment_processing.getType(), shipment_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));
        }

        //------------------------------------------出货反审核-------------------------
        else if (createReqVO.getApprovalType() == BoxApprovalTypeEnum.AIR_SHIPMENT_BACK.getType()) {
            flowKey = WorkFlowEmus.SHIPMENT_AUDIT_NO.getKey();

            //添加操作日志
            boxOpLogService.createBoxOpLog(boxDO.getId(), AirShipmentStatusEnum.SHIPMENT_BACK_APPROVAL_ING.getStatus(),
                    AirShipmentStatusEnum.SHIPMENT_BACK_APPROVAL_ING.getText(), AirShipmentStatusEnum.SHIPMENT_BACK_APPROVAL_ING.getTextEn(), boxApproval.getId());

            OrderApprovalTypeResultEnum shipment_back_processing = OrderApprovalTypeResultEnum.shipment_back_processing;
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                    null, shipment_back_processing.getType(), shipment_back_processing.getDesc(), Integer.parseInt(boxDO.getTransportType()));

        }

        //---------------------------------------撤销清关----------------------------
        else if(createReqVO.getApprovalType() == BoxApprovalTypeEnum.CLEARANCE_BACK.getType()) {
            if(boxDO.getClStatus() == ClStatueEnum.HAS_CLEARANCE.getClStatus()) {
                throw new ServiceException(500, "已清关，不能撤销");
            }
            boxService.checkIsInClearance(shipmentId, createReqVO.getOrderId());
            OrderApprovalTypeResultEnum clearance_back_processing = OrderApprovalTypeResultEnum.clearance_back_processing;
            boxService.updateOrderStatus(createReqVO.getOrderId(), null, null, null,
                    clearance_back_processing.getType(), clearance_back_processing.getDesc(), false);
            flowKey = WorkFlowEmus.SHIPMENT_CLEARANCE_STATUS_UPDATE.getKey();
            boxService.addOrderLog(Collections.singletonList(createReqVO.getOrderId()), OrderShipmentLog.CLEARANCE_BACK, "");
        }

        Long userId = SecurityFrameworkUtils.getLoginUserId();
        String bpmProcessId = bpmCreateServiceFactory.createBmp(userId,
                boxApproval.getId(), flowKey, boxDO.getSelfNo(),
                createReqVO.getCopyUserId());
        boxApproval.setBpmProcessId(bpmProcessId);
        boxApprovalMapper.updateById(boxApproval);
        // 返回
        return boxApproval.getId();
    }

    private void checkCustomsApproval(BoxDO boxDO) {
        Integer flyStatus = boxDO.getFlyStatus();
        if(flyStatus == FlyStatueEnum.FLYING.getDtStatus()) {
            throw new ServiceException(500, "出货已进行到下一步!");
        }
    }

    @Override
    public void updateBoxApproval(BoxApprovalUpdateReqVO updateReqVO) {
        // 更新
        BoxApprovalDO updateObj = BoxApprovalConvert.INSTANCE.convert(updateReqVO);
        boxApprovalMapper.updateById(updateObj);
    }

    @Override
    public void deleteBoxApproval(Long id) {
        // 删除
        boxApprovalMapper.deleteById(id);
    }


    @Override
    public BoxApprovalDO getBoxApproval(Long id) {
        return boxApprovalMapper.selectById(id);
    }

    @Override
    public List<BoxApprovalDO> getBoxApprovalList(Collection<Long> ids) {
        return boxApprovalMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxApprovalDO> getBoxApprovalPage(BoxApprovalQueryVO query, PageVO page) {
        return boxApprovalMapper.selectPage(page, query);
    }

    @Override
    public List<BoxApprovalDO> getBoxApprovalList(BoxApprovalQueryVO query) {
        return boxApprovalMapper.selectList(query);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxApproveResult(String flowKey,
                                       String id,
                                       int result) {

        BoxApprovalDO boxApprovalDO = boxApprovalMapper.selectById(id);

        TaskResp taskResp =
                bpmCreateServiceFactory.getLastTaskByProcessInstanceId(boxApprovalDO.getBpmProcessId());

        Long shipmentId = boxApprovalDO.getShipmentId();
        BoxDO boxDO = boxService.getBox(shipmentId);

        Integer orderStatus = null;
        Integer shipmentState = null;
        Integer inWarehouseState = null;
        Long orderId = boxApprovalDO.getOrderId();
        WorkFlowEmus workFlowEnum = WorkFlowEmus.keyOf(flowKey);
        if (Objects.isNull(workFlowEnum)) {
            throw exception(WORK_FLOW_NOT_EXITS);
        }

//        boxService.checkOrdersApprovaling(shipmentId);

        switch (workFlowEnum) {
            case SHIPMENT_PREASSEMBLE:
                //预装审批
                PrStatusEnum prStatusEnum = null;
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    prStatusEnum = PrStatusEnum.PRELOAD_APPROVAL_SUCCESS;

                    //审核成功修改订单状态改为已预装
                    orderStatus = OrderStatusEnum.PRE_INSTALLED.getValue();
                    shipmentState = BoxStatusEnum.PREINSTALL_EXAMINE_SUCCESS.getStatus();
                    inWarehouseState = 0;
                    boxService.generateBillLadingNo(shipmentId, false, true);

                    //修改订单的自编号
                    boxService.updateOrderSelfNoByShipmentId(shipmentId);

                    //修改出货的预装时间
                    boxDO.setYzDate(new Date());

                    //发送消息给通知人
                    boxService.sendPreloadMessage(shipmentId);
                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    //审核失败退回状态时需要改入仓子状态202合出货子状态0
                    prStatusEnum = PrStatusEnum.PRELOAD_APPROVAL_FAIL;
                    shipmentState = 0;//BoxStatusEnum.PREINSTALL_EXAMINE_FAIL.getStatus();
                    inWarehouseState = OrderWarehouseInStatusEnum.warehouse_finished.getValue();
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    prStatusEnum = PrStatusEnum.PRELOAD_ING;
                    shipmentState = 0;
                    inWarehouseState = OrderWarehouseInStatusEnum.warehouse_finished.getValue();
                }
                if (prStatusEnum != null) {
                    boxDO.setPrStatus(prStatusEnum.getPrStatus());
                    //取消审核不添加日志
                    if (prStatusEnum != PrStatusEnum.PRELOAD_ING) {
                        //新增审核日志
                        boxOpLogService.createBoxOpLog(shipmentId, prStatusEnum.getPrStatus(),
                                prStatusEnum.getStatusNameZh(), prStatusEnum.getStatusNameEn(), boxApprovalDO.getId());
                    }

                    //修改订单状态
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, orderStatus, inWarehouseState, shipmentState,
                            0, OrderApprovalTypeResultEnum.typeAndResultOf(18, result).getDesc(),
                            boxDO.getYzDate(), boxDO.getYzDate(), Integer.parseInt(boxDO.getTransportType()));
                }
                break;
            case SHIPMENT_PREASSEMBLE_NO:
                //预装反审
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    //返回成功，修改为预装中的状态
                    boxDO.setSlStatus(SlStatueEnum.NO_SETTLEMENT.getSlStatus());
                    boxDO.setUlStatus(ULStatusEnum.NO_CABINET_UNLOAD.getUlStatus());
                    boxDO.setSapStatus(SapStatueEnum.NO_ARRIVE.getSapStatus());
                    boxDO.setStkStatus(StkStatueEnum.NO_TAKE_OFF.getStkStatus());
                    boxDO.setClStatus(ClStatueEnum.NO_CLEARANCE.getClStatus());
                    boxDO.setApStatus(ApStatueEnum.NO_ARRIVE.getApStatus());
                    boxDO.setCdStatus(CdStatueEnum.NO_CLEARANCE_DOCUMENTS.getCdStatus());
                    boxDO.setCpStatus(CpStatueEnum.NO_UPLOAD.getCpStatus());
                    boxDO.setDtStatus(DtStatueEnum.NO_SHIPPING.getDtStatus());
                    boxDO.setBgStatus(BgStatueEnum.NO_BARGE.getBgStatus());
                    boxDO.setBlStatus(BlStatueEnum.NO_LADING_BILL.getBlStatus());
                    boxDO.setSaStatus(SaStatueEnum.NO_SHIP.getSaStatus());
                    boxDO.setDcStatus(DcStatueEnum.UNCUSTOMED.getDcStatus());
                    boxDO.setLdStatus(LdStatusEnum.NO_CABINET.getLdStatus());
                    boxDO.setTlStatus(TlStatueEnum.NO_SEND_CAR.getTlStatus());
                    boxDO.setTyStatus(TyStatueEnum.NO_TALLY.getTyStatus());
                    boxDO.setPrStatus(PrStatusEnum.PRELOAD_ING.getPrStatus());

                    //添加操作日志
                    boxOpLogService.createBoxOpLog(boxDO.getId(), PrStatusEnum.PRELOAD_BACK_APPROVAL_SUCCESS.getPrStatus(),
                            PrStatusEnum.PRELOAD_BACK_APPROVAL_SUCCESS.getStatusNameZh(), PrStatusEnum.PRELOAD_BACK_APPROVAL_SUCCESS.getStatusNameEn(), boxApprovalDO.getId());

                    //反审后修改订单为入仓状态
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.IN_WAREHOUSE.getValue(),
                            OrderWarehouseInStatusEnum.warehouse_finished.getValue(), 0, 0, OrderApprovalTypeResultEnum.typeAndResultOf(24, result).getDesc(), Integer.parseInt(boxDO.getTransportType()));

                    //反审后清除订单表的提单号
                    boxService.clearOrderTidanNo(shipmentId, false);

                    boxService.backOrderLog(boxDO.getTransportType(), boxDO.getSelfNo());
                    //清除卸柜箱数
                    boxService.clearUnloadCabinet(shipmentId);
//                boxService.clearLadingBill(shipmentId);

                    //业绩变更
                    boxService.addTargetLog(shipmentId, WITHDRAWAL, new Date());
                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    boxOpLogService.createBoxOpLog(boxDO.getId(), PrStatusEnum.PRELOAD_BACK_APPROVAL_FAIL.getPrStatus(),
                            PrStatusEnum.PRELOAD_BACK_APPROVAL_FAIL.getStatusNameZh(), PrStatusEnum.PRELOAD_BACK_APPROVAL_FAIL.getStatusNameEn(), boxApprovalDO.getId());
                    //反审失败
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(24, result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    //反审取消
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(24, result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                }
                break;
            case SHIPMENT_CLOSE_CONTAINER_NO:
                //封柜反审
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    //返回成功，修改为装柜中的状态
                    boxDO.setSlStatus(SlStatueEnum.NO_SETTLEMENT.getSlStatus());
                    boxDO.setUlStatus(ULStatusEnum.NO_CABINET_UNLOAD.getUlStatus());
                    boxDO.setSapStatus(SapStatueEnum.NO_ARRIVE.getSapStatus());
                    boxDO.setStkStatus(StkStatueEnum.NO_TAKE_OFF.getStkStatus());
                    boxDO.setClStatus(ClStatueEnum.NO_CLEARANCE.getClStatus());
                    boxDO.setApStatus(ApStatueEnum.NO_ARRIVE.getApStatus());
                    boxDO.setCdStatus(CdStatueEnum.NO_CLEARANCE_DOCUMENTS.getCdStatus());
                    boxDO.setCpStatus(CpStatueEnum.NO_UPLOAD.getCpStatus());
                    boxDO.setDtStatus(DtStatueEnum.NO_SHIPPING.getDtStatus());
                    boxDO.setBgStatus(BgStatueEnum.NO_BARGE.getBgStatus());
                    boxDO.setBlStatus(BlStatueEnum.NO_LADING_BILL.getBlStatus());
                    boxDO.setSaStatus(SaStatueEnum.NO_SHIP.getSaStatus());
                    boxDO.setDcStatus(DcStatueEnum.UNCUSTOMED.getDcStatus());
                    boxDO.setLdStatus(LdStatusEnum.IN_CABINET.getLdStatus());

                    //反审后修改订单为预装状态
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.PRE_INSTALLED.getValue(),
                            0, BoxStatusEnum.IN_THE_CABINETS.getStatus(), 0, OrderApprovalTypeResultEnum.typeAndResultOf(25, result).getDesc(), Integer.parseInt(boxDO.getTransportType()));

                    //封柜反审日志
                    boxOpLogService.createBoxOpLog(boxDO.getId(), LdStatusEnum.CABINET_BACK_EXAMINE_SUCCESS.getLdStatus(),
                            LdStatusEnum.CABINET_BACK_EXAMINE_SUCCESS.getStatusNameZh(), LdStatusEnum.CABINET_BACK_EXAMINE_SUCCESS.getStatusNameEn(), boxApprovalDO.getId());


                    //业绩变更
                    boxService.addTargetLog(shipmentId, WITHDRAWAL, new Date());
                    //清除卸柜箱数
                    boxService.clearUnloadCabinet(shipmentId);
                    // TODO 装柜反审短信发送
//                boxService.sendSms(shipmentId, SmsSceneEnum.SHIPMENT_CLOSE_CONTAINER);
                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    //封柜反审日志
                    boxOpLogService.createBoxOpLog(boxDO.getId(), LdStatusEnum.CABINET_BACK_EXAMINE_FAIL.getLdStatus(),
                            LdStatusEnum.CABINET_BACK_EXAMINE_FAIL.getStatusNameZh(), LdStatusEnum.CABINET_BACK_EXAMINE_FAIL.getStatusNameEn(), boxApprovalDO.getId());

                    //反审失败
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, 0,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(25, result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    //反审取消
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, 0,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(25, result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                }
                break;
            case SHIPMENT_CLOSE_CONTAINER:
                //封柜审核
                LdStatusEnum ldStatusEnum = null;
                orderStatus = OrderStatusEnum.INSTALLED_CABINET.getValue();
                Date sailTime = null; // 预计开船时间
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    ldStatusEnum = LdStatusEnum.CABINET_EXAMINE_SUCCESS;
                    shipmentState = BoxStatusEnum.SEAL_EXAMINE_SUCCESS.getStatus();
                    //修改为装柜审核后生成提单号
                    boxService.generateBillLadingNo(shipmentId, true, false);
                    BoxBookSeaDO boxBookSeaDO = boxBookSeaMapper.selectOne(new LambdaQueryWrapper<BoxBookSeaDO>().eq(BoxBookSeaDO::getShipmentId, shipmentId).last("limit 1"));
                    if (Objects.nonNull(boxBookSeaDO)){
                        sailTime = boxBookSeaDO.getSailTime();
                    }
                    //新建应收
                    boxService.buildSpecialReceivable(shipmentId);

                    //发送站内信
                    boxService.sendCabinetMessage(shipmentId);
                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    ldStatusEnum = LdStatusEnum.CABINET_EXAMINE_FAIL;
                    shipmentState = BoxStatusEnum.INSTALLED_WAIT_SEALED.getStatus();
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    ldStatusEnum = LdStatusEnum.INSTALLED_TO_BE_SEALED;
                    shipmentState = BoxStatusEnum.INSTALLED_WAIT_SEALED.getStatus();
                }
                if (ldStatusEnum != null) {
                    boxDO.setLdStatus(ldStatusEnum.getLdStatus());
                    if (ldStatusEnum != LdStatusEnum.INSTALLED_TO_BE_SEALED) {
                        //封柜审核日志
                        boxOpLogService.createBoxOpLog(shipmentId, ldStatusEnum.getLdStatus(),
                                ldStatusEnum.getStatusNameZh(), ldStatusEnum.getStatusNameEn(), boxApprovalDO.getId());
                    }

                    //修改订单状态
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, orderStatus, 0,
                            shipmentState, 0, OrderApprovalTypeResultEnum.typeAndResultOf(19, result).getDesc(),
                            boxDO.getZgDate(), sailTime, Integer.parseInt(boxDO.getTransportType()));
                }
                break;
            case SHIPMENT_UNLOAD_CONTAINER:
                //卸柜审核
                ULStatusEnum ulStatusEnum = null;
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    ulStatusEnum = ULStatusEnum.UNLOADING_EXAMINE_SUCCESS;
//                shipmentState = BoxStatusEnum.UNLOAD_EXAMINE_SUCCESS.getStatus();

                    //更新卸柜时间
                    boxService.updateUlBoxTime(shipmentId, new Date());
                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    ulStatusEnum = ULStatusEnum.UNLOADING_EXAMINE_FAIL;
                    orderStatus = OrderStatusEnum.CLEARED.getValue();
                    shipmentState = BoxStatusEnum.CLEARANCE_WAIT_UNLOAD.getStatus();
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    ulStatusEnum = ULStatusEnum.UNLOADING_CABINET;
                    orderStatus = OrderStatusEnum.CLEARED.getValue();
                    shipmentState = BoxStatusEnum.CLEARANCE_WAIT_UNLOAD.getStatus();
                }
                if (ulStatusEnum != null) {
                    boxDO.setUlStatus(ulStatusEnum.getUlStatus());
                    if (ulStatusEnum != ULStatusEnum.UNLOADING_CABINET) {
                        //卸柜审核日志
                        boxOpLogService.createBoxOpLog(shipmentId, ulStatusEnum.getUlStatus(),
                                ulStatusEnum.getStatusNameZh(), ulStatusEnum.getStatusNameEn(), boxApprovalDO.getId());
                    }

                    //修改订单状态
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, orderStatus, 0,
                            shipmentState, 0, OrderApprovalTypeResultEnum.typeAndResultOf(20, result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                }
                break;
            case SHIPMENT_UNLOAD_CONTAINER_NO:
                //卸柜反审
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    //改为卸柜中
                    boxDO.setSlStatus(SlStatueEnum.NO_SETTLEMENT.getSlStatus());
                    boxDO.setUlStatus(ULStatusEnum.UNLOADING_CABINET.getUlStatus());
                    String transportType = boxDO.getTransportType();
                    orderStatus = OrderStatusEnum.CLEARED.getValue();
                    shipmentState = BoxStatusEnum.CLEARANCE_WAIT_UNLOAD.getStatus();
                    if (transportType.equals(TransportTypeEnum.SEA_AIR)) {
                        orderStatus = OrderStatusEnum.TWP_WAY_ARRIVAL.getValue();
                        shipmentState = BoxStatusEnum.TWO_WAY_ARRIVAL.getStatus();
                    }
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, orderStatus, 0,
                            shipmentState, 0, OrderApprovalTypeResultEnum.typeAndResultOf(26, result).getDesc(), Integer.parseInt(boxDO.getTransportType()));

                    //卸柜反审核日志
                    boxOpLogService.createBoxOpLog(shipmentId, ULStatusEnum.UNLOADING_BACK_EXAMINE_SUCCESS.getUlStatus(),
                            ULStatusEnum.UNLOADING_BACK_EXAMINE_SUCCESS.getStatusNameZh(), ULStatusEnum.UNLOADING_BACK_EXAMINE_SUCCESS.getStatusNameEn(), boxApprovalDO.getId());

                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    //卸柜反审核日志
                    boxOpLogService.createBoxOpLog(shipmentId, ULStatusEnum.UNLOADING_BACK_EXAMINE_FAIL.getUlStatus(),
                            ULStatusEnum.UNLOADING_BACK_EXAMINE_FAIL.getStatusNameZh(), ULStatusEnum.UNLOADING_BACK_EXAMINE_FAIL.getStatusNameEn(), boxApprovalDO.getId());

                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, 0,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(26, result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, 0,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(26, result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                }
                break;
            case SHIPMENT_CUSTOMS_EXIT_ALL:
                //报关全退
                boxDO.setCheckExamineStatus(result);
                int opStatus = Integer.parseInt(DcStatueEnum.CUSTOMING.getDcStatus() + "1" + result);
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    //如果审核通过，重新修改状态为未处理
                    boxDO.setCheckDealStatus(0);
                    boxOpLogService.createBoxOpLog(boxDO.getId(), opStatus, "报关全部退场审核成功", "Full return of customs declaration under review success", boxApprovalDO.getId());
                } else if(result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    boxOpLogService.createBoxOpLog(boxDO.getId(), opStatus, "报关全部退场审核失败", "Full return of customs declaration under review fail", boxApprovalDO.getId());
                }
                boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, 0,
                        null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(27, result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                break;
            case SHIPMENT_CUSTOMS_EXIT_PART:
                //报关部分退
                boxDO.setCheckExamineStatus(result);
                opStatus = Integer.parseInt(DcStatueEnum.CUSTOMING.getDcStatus() + "2" + result);
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    //如果审核通过，重新修改状态为未处理
                    boxDO.setCheckDealStatus(0);
                    boxOpLogService.createBoxOpLog(boxDO.getId(), opStatus, "报关部分退场审核成功", "part return of customs declaration under review success", boxApprovalDO.getId());
                } else if(result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    boxOpLogService.createBoxOpLog(boxDO.getId(), opStatus, "报关部分退场审核失败", "part return of customs declaration under review fail", boxApprovalDO.getId());
                }
                boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, 0,
                        null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(27, result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                break;
            case SHIPMENT_CUSTOMS_DECLARE_REMOVE_SPLIT:
                // 删单退场拆单业务(主要操作流程与部分退场拆单一致)
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    OrderSplitApplyDetailDto orderSplitApplyDetailDto =
                            JSON.parseObject(boxApprovalDO.getDetails(),
                                    OrderSplitApplyDetailDto.class);

                    OrderDO oldOrder = boxService.getOrder(orderId);
                    orderSplitApplyDetailDto.setBoxSplitOrder(true);// 标识为出货拆单，需要保持主拆单原状态、原单价等数据
                    orderSplitApplyDetailDto.setPartRefundSplit(true);// 标识为部分退场拆单，需要将子状态也保持原有状态(注：删单退场拆单逻辑一致，拆单后状态与母订单一致，等操作退场提交审批后才会修改状态)
                    boxService.splitSuccess(orderSplitApplyDetailDto, taskResp.getComment());

                    List<OrderSplitBackVO> orderSplitList = orderSplitApplyDetailDto.getOrderSplitBackVOList();
                    // 1就是加业绩  2是删业绩
                    // 删除原单业绩
                    // TODO 这里的拆单操作并没有正真的退场，是否在拆单完成后，提交退场确认操作时在进行删除目标业绩
                    targetLogService.addTargetLog(orderId, new Date(), WITHDRAWAL);
                    String masterOrderNo = "";
                    //没有拆单数据
                    if (CollectionUtil.isNotEmpty(orderSplitList)) {
                        //没有拆单数据
                        List<OrderDO> splitOrderList = new ArrayList<>();
                        for (OrderSplitBackVO orderSplit : orderSplitList) {
                            // 拆单通过后标记拆单缓存记录已使用
                            boxService.splitUsed(orderSplit.getId());
                            //拆出后的订单编号
                            String orderNo = orderSplit.getOrderNo();
                            OrderDO splitOrderDO = boxService.getSplitBoxOrder(orderNo);

                            if (orderSplit.getIsMaster()) {
                                //查出新拆出来的订单，出货修改为新的订单号（取第一个订单的订单号）
                                splitOrderDO.setIsMaster(true);
                                masterOrderNo = orderNo;
                                // 添加主拆单业绩
                                targetLogService.addTargetLog(orderId, new Date(), LOADING);
                            } else {
                                splitOrderDO.setIsMaster(false);
                            }
                            splitOrderList.add(splitOrderDO);
                        }
                        // 部分退场拆单业务待完善 修改出货单中的订单ID
                        boxService.updateExitSplitOrderId(shipmentId, oldOrder, splitOrderList);
                    }
                    // 需要修改原订单的出货状态和业绩
                    if (!StringUtils.equals(oldOrder.getOrderNo(), masterOrderNo)) {
                        // 当拆单订单与母订单编号不同，需要重置母订单的货柜自编号和提单编号
                        boxService.resetOrderTidanNo(orderId, false);
                    }
                    // 将原订单从退场订单列表删除
                    boxOrderExitService.remove(new LambdaQueryWrapper<BoxOrderExitDO>()
                            .eq(BoxOrderExitDO::getOrderId, orderSplitApplyDetailDto.getOrderId())
                            .eq(BoxOrderExitDO::getShipmentId, shipmentId));
                    //删除原订单的提单
                    boxService.removeLadingBill(Collections.singletonList(oldOrder.getOrderId()));
                    boxService.addOrderLog(Collections.singletonList(orderId), OrderShipmentLog.DELETE_ORDER_REFUND_SPLIT_RESULT_SUCCESS, "");
                    boxService.createCustomsSplitNotify(shipmentId, Collections.singletonList(orderSplitApplyDetailDto.getOrderId()), 1);
                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    // 部分退场拆单中的订单标记为待拆单
                    boxOrderExitService.update(new LambdaUpdateWrapper<BoxOrderExitDO>()
                            .set(BoxOrderExitDO::getSplitStatus, 1)
                            .eq(BoxOrderExitDO::getOrderId, orderId)
                            .eq(BoxOrderExitDO::getShipmentId, shipmentId));
                    //更新订单状态  入仓-拆单-已拒绝
                    boxService.deleteSplitItem(orderId);
                    boxService.updateStatus(orderId, null, null, null, null, null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(6, result).getDesc());
                    boxService.addOrderLog(Collections.singletonList(orderId), OrderShipmentLog.DELETE_ORDER_REFUND_SPLIT_RESULT_FAIL, "");
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    // 部分退场拆单中的订单标记为待拆单
                    boxOrderExitService.update(new LambdaUpdateWrapper<BoxOrderExitDO>()
                            .set(BoxOrderExitDO::getSplitStatus, 1)
                            .eq(BoxOrderExitDO::getOrderId, orderId)
                            .eq(BoxOrderExitDO::getShipmentId, shipmentId));
                    //更新订单状态  入仓-拆单-已取消
                    //删除拆单草稿信息
                    boxService.deleteSplitItem(orderId);
                    boxService.updateStatus(orderId, null, null, null, null, null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(6, result).getDesc());
                    boxService.addOrderLog(Collections.singletonList(orderId), OrderShipmentLog.DELETE_ORDER_REFUND_SPLIT_RESULT_FAIL, "");
                }

                break;
            case AIR_SPLIT_ORDER:
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    OrderSplitApplyDetailDto orderSplitApplyDetailDto = JSON.parseObject(boxApprovalDO.getDetails(), OrderSplitApplyDetailDto.class);
                    OrderDO oldOrder = boxService.getOrder(orderId);
                    orderSplitApplyDetailDto.setBoxSplitOrder(true);// 标识为出货拆单，需要保持原状态、原单价等数据
                    boxService.splitSuccess(orderSplitApplyDetailDto, taskResp.getComment());

                    List<OrderSplitBackVO> orderSplitList = orderSplitApplyDetailDto.getOrderSplitBackVOList();
                    //没有拆单数据
                    String masterOrderNo = "";
                    if (CollectionUtil.isNotEmpty(orderSplitList)) {
                        for (OrderSplitBackVO orderSplit : orderSplitList) {
                            // 拆单通过后标记拆单缓存记录已使用
                            boxService.splitUsed(orderSplit.getId());
                            if (orderSplit.getIsMaster()) {
                                //拆出后主订单的订单编号
                                masterOrderNo = orderSplit.getOrderNo();
                                //查出新拆出来的订单，出货修改为新的订单号（取第一个订单的订单号）
                                OrderDO splitOrderDO = boxService.getSplitBoxOrder(masterOrderNo);
                                //修改出货单中的订单ID
                                boxService.updateSplitOrderId(shipmentId, oldOrder, splitOrderDO.getOrderId(), masterOrderNo);
                            }
                        }
                    }
                    // 加入空运拆单业务后，判断是否需要重置母订单自编号和提单编号的逻辑根据母订单是否保存为新的拆单订单
                    if (!StringUtils.equals(oldOrder.getOrderNo(), masterOrderNo)) {
                        // 当拆单订单与母订单编号不同，需要重置母订单的货柜自编号和提单编号
                        boxService.resetOrderTidanNo(orderId, false);
                    }
                    //删除原订单的提单
                    boxService.removeLadingBill(Collections.singletonList(oldOrder.getOrderId()));
                    boxService.addOrderLog(Collections.singletonList(orderId), OrderShipmentLog.CABINET_SPLIT_RESULT_SUCCESS, "");
                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    //更新订单状态  入仓-拆单-已拒绝
                    boxService.updateStatus(orderId, null, null, null, null, null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(6, result).getDesc());
                    boxService.addOrderLog(Collections.singletonList(orderId), OrderShipmentLog.CABINET_SPLIT_RESULT_FAIL, "");
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    //更新订单状态  入仓-拆单-已取消
                    boxService.updateStatus(orderId, null, null, null, null, null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(6, result).getDesc());
//                boxService.splitCancelByOrderId(orderId);
                }
                //删除拆单草稿信息
                boxService.deleteSplitItem(orderId);
                break;
            case SHIPMENT_SPLIT_ORDER:
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    OrderSplitApplyDetailDto orderSplitApplyDetailDto = JSON.parseObject(boxApprovalDO.getDetails(), OrderSplitApplyDetailDto.class);
                    OrderDO oldOrder = boxService.getOrder(orderId);
                    orderSplitApplyDetailDto.setBoxSplitOrder(true);// 标识为出货拆单，需要保持原状态、原单价等数据
                    boxService.splitSuccess(orderSplitApplyDetailDto, taskResp.getComment());

                    List<OrderSplitBackVO> orderSplitList = orderSplitApplyDetailDto.getOrderSplitBackVOList();
                    //没有拆单数据
                    String masterOrderNo = "";
                    if (CollectionUtil.isNotEmpty(orderSplitList)) {
                        for (OrderSplitBackVO orderSplit : orderSplitList) {
                            // 拆单通过后标记拆单缓存记录已使用
                            boxService.splitUsed(orderSplit.getId());
                            if (orderSplit.getIsMaster()) {
                                //拆出后主订单的订单编号
                                masterOrderNo = orderSplit.getOrderNo();
                                //查出新拆出来的订单，出货修改为新的订单号（取第一个订单的订单号）
                                OrderDO splitOrderDO = boxService.getSplitBoxOrder(masterOrderNo);
                                //修改出货单中的订单ID
                                boxService.updateSplitOrderId(shipmentId, oldOrder, splitOrderDO.getOrderId(), masterOrderNo);
                            }
                        }
                    }
                    // 加入空运拆单业务后，判断是否需要重置母订单自编号和提单编号的逻辑根据母订单是否保存为新的拆单订单
                    if (!StringUtils.equals(oldOrder.getOrderNo(), masterOrderNo)) {
                        // 当拆单订单与母订单编号不同，需要重置母订单的货柜自编号和提单编号
                        boxService.resetOrderTidanNo(orderId, false);
                    }
                    //删除原订单的提单
                    boxService.removeLadingBill(Collections.singletonList(oldOrder.getOrderId()));
                    boxService.addOrderLog(Collections.singletonList(orderId), OrderShipmentLog.CABINET_SPLIT_RESULT_SUCCESS, "");
                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    //更新订单状态  入仓-拆单-已拒绝
                    boxService.updateStatus(orderId, null, null, null, null, null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(6, result).getDesc());
                    boxService.addOrderLog(Collections.singletonList(orderId), OrderShipmentLog.CABINET_SPLIT_RESULT_FAIL, "");
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    //更新订单状态  入仓-拆单-已取消
                    boxService.updateStatus(orderId, null, null, null, null, null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(6, result).getDesc());
//                boxService.splitCancelByOrderId(orderId);
                }
                //删除拆单草稿信息
                boxService.deleteSplitItem(orderId);
                break;
            case SHIPMENT_EXIT_PART_SPILT:
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    OrderSplitApplyDetailDto orderSplitApplyDetailDto = JSON.parseObject(boxApprovalDO.getDetails(), OrderSplitApplyDetailDto.class);
                    OrderDO oldOrder = boxService.getOrder(orderId);
                    orderSplitApplyDetailDto.setBoxSplitOrder(true);// 标识为出货拆单，需要保持主拆单原状态、原单价等数据
                    orderSplitApplyDetailDto.setPartRefundSplit(true);// 标识为部分退场拆单，需要将子状态也保持原有状态
                    boxService.splitSuccess(orderSplitApplyDetailDto, taskResp.getComment());

                    List<OrderSplitBackVO> orderSplitList = orderSplitApplyDetailDto.getOrderSplitBackVOList();
                    // 1就是加业绩  2是删业绩
                    // 删除原单业绩
                    targetLogService.addTargetLog(orderId, new Date(), WITHDRAWAL);
                    String masterOrderNo = "";
                    //没有拆单数据
                    if (CollectionUtil.isNotEmpty(orderSplitList)) {
                        //没有拆单数据
                        List<OrderDO> splitOrderList = new ArrayList<>();
                        for (OrderSplitBackVO orderSplit : orderSplitList) {
                            // 拆单通过后标记拆单缓存记录已使用
                            boxService.splitUsed(orderSplit.getId());
                            //拆出后的订单编号
                            String orderNo = orderSplit.getOrderNo();
                            OrderDO splitOrderDO = boxService.getSplitBoxOrder(orderNo);

                            if (orderSplit.getIsMaster()) {
                                //查出新拆出来的订单，出货修改为新的订单号（取第一个订单的订单号）
                                splitOrderDO.setIsMaster(true);
                                masterOrderNo= orderNo;
                                // 添加主拆单业绩
                                targetLogService.addTargetLog(orderId, new Date(), LOADING);
                            } else {
                                splitOrderDO.setIsMaster(false);
                            }
                            splitOrderList.add(splitOrderDO);
                        }
                        // 部分退场拆单业务待完善 修改出货单中的订单ID
                        boxService.updateExitSplitOrderId(shipmentId, oldOrder, splitOrderList);
                    }
                    // 需要修改原订单的出货状态和业绩
                    if (!StringUtils.equals(oldOrder.getOrderNo(), masterOrderNo)) {
                        // 当拆单订单与母订单编号不同，需要重置母订单的货柜自编号和提单编号
                        boxService.resetOrderTidanNo(orderId, false);
                    }
                    // 将原订单从退场订单列表删除
                    boxOrderExitService.remove(new LambdaQueryWrapper<BoxOrderExitDO>()
                            .eq(BoxOrderExitDO::getOrderId, orderSplitApplyDetailDto.getOrderId())
                            .eq(BoxOrderExitDO::getShipmentId, shipmentId));
                    boxService.addOrderLog(Collections.singletonList(orderId), OrderShipmentLog.PART_REFUND_SPLIT_RESULT_SUCCESS, "");
                    boxService.createCustomsSplitNotify(shipmentId, Collections.singletonList(orderSplitApplyDetailDto.getOrderId()), 0);
                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    // 部分退场拆单中的订单标记为待拆单
                    boxOrderExitService.update(new LambdaUpdateWrapper<BoxOrderExitDO>()
                            .set(BoxOrderExitDO::getSplitStatus, 1)
                            .eq(BoxOrderExitDO::getOrderId, orderId)
                            .eq(BoxOrderExitDO::getShipmentId, shipmentId));
                    //更新订单状态  入仓-拆单-已拒绝
                    boxService.deleteSplitItem(orderId);
                    boxService.updateStatus(orderId, null, null, null, null, null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(6, result).getDesc());
                    boxService.addOrderLog(Collections.singletonList(orderId), OrderShipmentLog.PART_REFUND_SPLIT_RESULT_FAIL, "");
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    // 部分退场拆单中的订单标记为待拆单
                    boxOrderExitService.update(new LambdaUpdateWrapper<BoxOrderExitDO>()
                            .set(BoxOrderExitDO::getSplitStatus, 1)
                            .eq(BoxOrderExitDO::getOrderId, orderId)
                            .eq(BoxOrderExitDO::getShipmentId, shipmentId));
                    //更新订单状态  入仓-拆单-已取消
                    //删除拆单草稿信息
                    boxService.deleteSplitItem(orderId);
                    boxService.updateStatus(orderId, null, null, null, null, null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(6, result).getDesc());
                    boxService.addOrderLog(Collections.singletonList(orderId), OrderShipmentLog.PART_REFUND_SPLIT_RESULT_FAIL, "");
                }

                break;

                //===================================空运=======================================
            //======================批量加价格=======================
            case SHIPMENT_BATCH_ADD_PRICE:
                //批量加价格
                // 添加注释 是给自编号下的订单加价
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    String details = boxApprovalDO.getDetails();
                    BoxOrderMarkUpVO boxOrderMarkUpVO =
                            JSON.parseObject(details, BoxOrderMarkUpVO.class);
                    if(boxOrderMarkUpVO.getOrderIds()!=null)
                    {
                        String s=String.valueOf(boxOrderMarkUpVO.getOrderIds());
                        zTest z=new zTest();
                        z.setTestname("订单id，"+s);
                        z.setCreatedate(new Date());
                        zTestMapper2.insert(z);
                    }
                    else
                    {
                        String s=String.valueOf(boxOrderMarkUpVO.getOrderIds());
                        zTest z=new zTest();
                        z.setTestname("批量加价申报批，订单id为空");
                        z.setCreatedate(new Date());
                        zTestMapper2.insert(z);
                    }
                    boxService.checkOrderPayStatus(boxOrderMarkUpVO.getOrderIds());
                    boxService.batchAddPriceSuccess(shipmentId, details);
                }
                break;

                //==============删单退场=============================
            case SHIPMENT_CUSTOMS_DECLARE_REMOVE:
                //解析审核详情
                //出货审批-报关删单退场审核
                CustomsExitType customsExitType =
                        JSON.parseObject(boxApprovalDO.getDetails(),
                                CustomsExitType.class);
                if(customsExitType.getDeleteExitType() == 2) {
                    opStatus = Integer.parseInt(DcStatueEnum.CUSTOMING.getDcStatus()
                            + "3" + result);
                    if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                        //如果审核通过，重新修改状态为未处理
                        boxDO.setCheckDealStatus(0);
                        boxOpLogService.createBoxOpLog(boxDO.getId(),
                                opStatus, "报关删单退场审核成功",
                                "delete return of customs declaration under review success",
                                boxApprovalDO.getId());
                    } else if(result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                        boxOpLogService.createBoxOpLog(boxDO.getId(), opStatus,
                                "报关删单审核失败",
                                "delete return of customs declaration under review fail",
                                boxApprovalDO.getId());
                    }
                }
                boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                        null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(27, result).getDesc(), Integer.parseInt(boxDO.getTransportType()));


                boxDO.setOverMachineExamineStatus(result);
                break;

            //====================排单审核======================
            case SHIPMENT_ORDER_SORTING:
                //排单审核
                StStatusEnum stStatusEnum = null;
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    stStatusEnum = StStatusEnum.SORTING_APPROVAL_SUCCESS;

                    //审核成功修改订单状态改为已预装
                    orderStatus = OrderStatusEnum.PRE_INSTALLED.getValue();
                    shipmentState = BoxAirStatusEnum.SORTING_EXAMINE_SUCCESS.getStatus();
                    inWarehouseState = 0;
                    boxService.generateBillLadingNo(shipmentId, true, false);

                    //修改订单的自编号
                    boxService.updateOrderSelfNoByShipmentId(shipmentId);

                    //锁定价格
                    boxService.lockOrderPrice(shipmentId);

                    boxService.buildSpecialReceivable(shipmentId);

                    //修改出货的预装时间
                    boxDO.setYzDate(new Date());

                    //发送消息给通知人
                    boxService.sendPreloadMessage(shipmentId);
                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    //审核失败退回状态时需要改入仓子状态202合出货子状态0
                    stStatusEnum = StStatusEnum.SORTING_APPROVAL_FAIL;
                    shipmentState = 0;
                    inWarehouseState = OrderWarehouseInStatusEnum.warehouse_finished.getValue();
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    stStatusEnum = StStatusEnum.SORTING;
                    shipmentState = 0;
                    inWarehouseState = OrderWarehouseInStatusEnum.warehouse_finished.getValue();
                }
                if (stStatusEnum != null) {
                    boxDO.setPrStatus(stStatusEnum.getStStatus());
                    //取消审核不添加日志
                    if (stStatusEnum != StStatusEnum.SORTING) {
                        //新增审核日志
                        boxOpLogService.createBoxOpLog(shipmentId, stStatusEnum.getStStatus(),
                                stStatusEnum.getStatusNameZh(), stStatusEnum.getStatusNameEn(), boxApprovalDO.getId());
                    }

                    //修改订单状态
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, orderStatus, inWarehouseState,
                            shipmentState, 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.sorting_pass.getType(), result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                }
                break;

            //========================排单反审==========================
            case SHIPMENT_ORDER_SORTING_NO:
                //排单反审
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    //返回成功，修改为预装中的状态
                    boxDO.setSlStatus(SlStatueEnum.NO_SETTLEMENT.getSlStatus());
                    boxDO.setToWarehouseStatus(TWStatusEnum.NO_TO_WAREHOUSE.getTwStatus());
                    boxDO.setClStatus(ClStatueEnum.NO_CLEARANCE.getClStatus());
                    boxDO.setSapStatus(SapStatueEnum.NO_ARRIVE.getSapStatus());
                    boxDO.setCdStatus(CdStatueEnum.NO_CLEARANCE_DOCUMENTS.getCdStatus());
                    boxDO.setFlyStatus(FlyStatueEnum.NO_FLYING.getDtStatus());
                    boxDO.setBlStatus(BlStatueEnum.NO_LADING_BILL.getBlStatus());
                    boxDO.setDcStatus(DcStatueEnum.UNCUSTOMED.getDcStatus());
                    boxDO.setCheckoutStatus(AirCheckoutStatusEnum.UNCHECKOUT.getStatus());
                    boxDO.setAirShipmentStatus(AirShipmentStatusEnum.UNSHIPMENT.getStatus());
                    boxDO.setMergePkgStatus(MergePkgStatusEnum.UNMERGE.getStatus());
                    boxDO.setTyStatus(TyStatueEnum.NO_TALLY.getTyStatus());
                    boxDO.setPrStatus(StStatusEnum.SORTING.getStStatus());

                    //添加操作日志
                    boxOpLogService.createBoxOpLog(boxDO.getId(), StStatusEnum.SORTING_BACK_APPROVAL_SUCCESS.getStStatus(),
                            StStatusEnum.SORTING_BACK_APPROVAL_SUCCESS.getStatusNameZh(), StStatusEnum.SORTING_BACK_APPROVAL_SUCCESS.getStatusNameEn(), boxApprovalDO.getId());

                    //反审后修改订单为入仓状态
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.IN_WAREHOUSE.getValue(),
                            OrderWarehouseInStatusEnum.warehouse_finished.getValue(), 0, 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.sorting_back_pass.getType(), result).getDesc(), Integer.parseInt(boxDO.getTransportType()));

                    //反审后清除订单表的提单号
                    boxService.clearOrderTidanNo(shipmentId, false);

                    //解锁价格
                    boxService.unlockOrderPrice(shipmentId);

                    //业绩变更
                    boxService.addTargetLog(shipmentId, WITHDRAWAL, new Date());

                    //删除已到港订单
                    boxService.deleteArrivalOrderByShipment(shipmentId);
                    //删除已清关订单
                    boxService.deleteClearanceOrderByShipment(shipmentId);

                    boxService.backOrderLog(boxDO.getTransportType(), boxDO.getSelfNo());

                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    boxOpLogService.createBoxOpLog(boxDO.getId(), StStatusEnum.SORTING_BACK_APPROVAL_FAIL.getStStatus(),
                            StStatusEnum.SORTING_BACK_APPROVAL_FAIL.getStatusNameZh(), StStatusEnum.SORTING_BACK_APPROVAL_FAIL.getStatusNameEn(), boxApprovalDO.getId());
                    //反审失败
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.sorting_back_pass.getType(), result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    //反审取消
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.sorting_back_pass.getType(), result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                }

                break;

                //===========================到仓审核========================
            case SHIPMENT_TO_WAREHOUSE:
                TWStatusEnum twStatusEnum = null;
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    twStatusEnum = TWStatusEnum.TO_WAREHOUSED_SEALED;
                    shipmentState = BoxAirStatusEnum.TO_WAREHOUSED.getStatus();
                    orderStatus = OrderStatusEnum.UNLOADED_CABINET.getValue();
                    //创建提货异常
                    boxService.createPickUpException(shipmentId);
                    //创建代收货款异常
                    boxService.createPaymentException(shipmentId);
                    // 空运站内信
                    boxService.shipmentAirInternalMessage(shipmentId, boxDO.getSelfNo(), null, BoxAirStatusEnum.TO_WAREHOUSED, Integer.parseInt(boxDO.getTransportType()));
                    //更新到仓时间
                    boxService.updateUlBoxTime(shipmentId, new Date());
                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    twStatusEnum = TWStatusEnum.TO_WAREHOUSE_EXAMINE_FAIL;
                    orderStatus = OrderStatusEnum.CLEARED.getValue();
                    shipmentState = BoxStatusEnum.CLEARANCE_WAIT_UNLOAD.getStatus();
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    twStatusEnum = TWStatusEnum.TO_WAREHOUSING;
                    orderStatus = OrderStatusEnum.CLEARED.getValue();
                    shipmentState = BoxStatusEnum.CLEARANCE_WAIT_UNLOAD.getStatus();
                }
                if (twStatusEnum != null) {
                    boxDO.setToWarehouseStatus(twStatusEnum.getTwStatus());
                    if (twStatusEnum != TWStatusEnum.TO_WAREHOUSING) {
                        //到仓审核日志
                        boxOpLogService.createBoxOpLog(shipmentId, twStatusEnum.getTwStatus(),
                                twStatusEnum.getStatusNameZh(), twStatusEnum.getStatusNameEn(), boxApprovalDO.getId());
                    }

                    //修改订单状态
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, orderStatus, 0,
                            shipmentState, 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.warehoused_pass.getType(), result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                }
                break;

                //=======================到仓反审========================
            case SHIPMENT_TO_WAREHOUSE_No:
                //卸柜反审
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    //改为到仓中
                    boxDO.setSlStatus(SlStatueEnum.NO_SETTLEMENT.getSlStatus());
                    boxDO.setToWarehouseStatus(TWStatusEnum.TO_WAREHOUSING.getTwStatus());
                    orderStatus = OrderStatusEnum.CLEARED.getValue();
                    shipmentState = BoxStatusEnum.CLEARANCE_WAIT_UNLOAD.getStatus();

                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, orderStatus, 0,
                            shipmentState, 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.warehoused_back_pass.getType(), result).getDesc(), Integer.parseInt(boxDO.getTransportType()));

                    //到仓反审核日志
                    boxOpLogService.createBoxOpLog(shipmentId, TWStatusEnum.TO_WAREHOUSE_BACK_EXAMINE_SUCCESS.getTwStatus(),
                            TWStatusEnum.TO_WAREHOUSE_BACK_EXAMINE_SUCCESS.getStatusNameZh(), TWStatusEnum.TO_WAREHOUSE_BACK_EXAMINE_SUCCESS.getStatusNameEn(), boxApprovalDO.getId());

                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    //到仓反审核日志
                    boxOpLogService.createBoxOpLog(shipmentId, TWStatusEnum.TO_WAREHOUSE_BACK_EXAMINE_FAIL.getTwStatus(),
                            TWStatusEnum.TO_WAREHOUSE_BACK_EXAMINE_FAIL.getStatusNameZh(), TWStatusEnum.TO_WAREHOUSE_BACK_EXAMINE_FAIL.getStatusNameEn(), boxApprovalDO.getId());

                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.warehoused_back_pass.getType(), result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.warehoused_back_pass.getType(), result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                }
                break;

                //====================================出货审核===========================================
            case SHIPMENT_AUDIT:
                AirShipmentStatusEnum airShipmentStatusEnum = null;
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    airShipmentStatusEnum = AirShipmentStatusEnum.SHIPMENT_APPROVAL_SUCCESS;

                    boxDO.setAirShipmentStatus(AirShipmentStatusEnum.SHIPMENT.getStatus());
                    //修改为出货审核后生成提单号
//                    boxService.generateBillLadingNo(shipmentId, true, false);
                    //修改订单状态
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.SHIPMENT.getValue(),
                            null, BoxAirStatusEnum.AIR_SHIPPED.getStatus(), 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.shipment_pass.getType(), result).getDesc(), Integer.parseInt(boxDO.getTransportType()));

                    boxService.addOrderLogByShipmentId(shipmentId, OrderShipmentLog.AIR_OUT_SHIPMENT, "");
                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    airShipmentStatusEnum = AirShipmentStatusEnum.SHIPMENT_APPROVAL_FAIL;

                    //修改订单状态
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.shipment_reject.getType(), result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    airShipmentStatusEnum = AirShipmentStatusEnum.UNSHIPMENT;
                    //修改订单状态
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.shipment_cancel.getType(), result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                }
                if (result != BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    //新增审核日志
                    boxOpLogService.createBoxOpLog(shipmentId, airShipmentStatusEnum.getStatus(),
                            airShipmentStatusEnum.getText(), airShipmentStatusEnum.getTextEn(), boxApprovalDO.getId());
                }
                break;

            //========================出货反审==========================
            case SHIPMENT_AUDIT_NO:
                //出货反审
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    //返回成功，修改为预装中的状态
                    boxDO.setSlStatus(SlStatueEnum.NO_SETTLEMENT.getSlStatus());
                    boxDO.setToWarehouseStatus(TWStatusEnum.NO_TO_WAREHOUSE.getTwStatus());
                    boxDO.setClStatus(ClStatueEnum.NO_CLEARANCE.getClStatus());
                    boxDO.setSapStatus(SapStatueEnum.NO_ARRIVE.getSapStatus());
                    boxDO.setCdStatus(CdStatueEnum.NO_CLEARANCE_DOCUMENTS.getCdStatus());
                    boxDO.setFlyStatus(FlyStatueEnum.NO_FLYING.getDtStatus());
                    boxDO.setBlStatus(BlStatueEnum.NO_LADING_BILL.getBlStatus());
                    boxDO.setDcStatus(DcStatueEnum.UNCUSTOMED.getDcStatus());
                    boxDO.setCheckoutStatus(AirCheckoutStatusEnum.UNCHECKOUT.getStatus());
                    boxDO.setAirShipmentStatus(AirShipmentStatusEnum.UNSHIPMENT.getStatus());
                    boxDO.setShipmentStatusAir(BoxAirStatusEnum.TALLY_COMPLETE.getStatus());
                    //添加操作日志
                    boxOpLogService.createBoxOpLog(boxDO.getId(), AirShipmentStatusEnum.SHIPMENT_BACK_APPROVAL_SUCCESS.getStatus(),
                            AirShipmentStatusEnum.SHIPMENT_BACK_APPROVAL_SUCCESS.getText(), AirShipmentStatusEnum.SHIPMENT_BACK_APPROVAL_SUCCESS.getTextEn(), boxApprovalDO.getId());

                    //反审后修改订单为出货状态
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.SHIPMENT.getValue(), null,
                            BoxAirStatusEnum.TALLY_COMPLETE.getStatus(), 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.shipment_back_pass.getType(), result).getDesc(), Integer.parseInt(boxDO.getTransportType()));

                    //反审后清除订单表的提单号
//                    boxService.clearOrderTidanNo(shipmentId, false);

                    //业绩变更
                    boxService.addTargetLog(shipmentId, WITHDRAWAL, new Date());

                    //删除已到港订单
                    boxService.deleteArrivalOrderByShipment(shipmentId);
                    //删除已清关订单
                    boxService.deleteClearanceOrderByShipment(shipmentId);

                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    boxOpLogService.createBoxOpLog(boxDO.getId(), AirShipmentStatusEnum.SHIPMENT_BACK_APPROVAL_FAIL.getStatus(),
                            AirShipmentStatusEnum.SHIPMENT_BACK_APPROVAL_FAIL.getText(), AirShipmentStatusEnum.SHIPMENT_BACK_APPROVAL_FAIL.getTextEn(), boxApprovalDO.getId());
                    //反审失败
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.shipment_back_pass.getType(), result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    //反审取消
                    boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.shipment_back_pass.getType(), result).getDesc(), Integer.parseInt(boxDO.getTransportType()));
                }

                break;

                //=========================撤销清关=============================
            case SHIPMENT_CLEARANCE_STATUS_UPDATE:
                Long backOrderId = boxApprovalDO.getOrderId();
                if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
                    //状态回退
                    boxService.updateOrderStatusByOrderList(Collections.singletonList(backOrderId), OrderStatusEnum.TWP_WAY_ARRIVAL.getValue(),
                            null, BoxAirStatusEnum.ARRIVAL_WAIT_DISCHARGE.getStatus(),
                            0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.clearance_back_pass.getType(), result).getDesc(), null, boxDO.getDgDate());
                    //删除已清关订单
                    boxService.deleteClearanceOrder(shipmentId, backOrderId);
                } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
                    //反审失败
                    boxService.updateOrderStatusByOrderList(Collections.singletonList(backOrderId), null, null,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.clearance_back_reject.getType(), result).getDesc(), null, null);

                } else if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) {
                    //反审失败
                    boxService.updateOrderStatusByOrderList(Collections.singletonList(backOrderId), null, null,
                            null, 0, OrderApprovalTypeResultEnum.typeAndResultOf(OrderApprovalTypeResultEnum.clearance_back_cancel.getType(), result).getDesc(), null, null);

                }
                break;

        }
        boxService.updateById(boxDO);

        boxApprovalDO.setApprovalStatus(result);
        updateById(boxApprovalDO);

//        if (result == BpmProcessInstanceResultEnum.CANCEL.getResult()) return;
//        String approvalType = WorkFlowEmus.getValue(flowKey);
//        approvalType = approvalType.replace("出货审批-", "");
//        String selfNo = boxDO.getSelfNo();
//        sendMessage(approvalType, selfNo, taskResp, Long.valueOf(boxApprovalDO.getCreator()));
    }

    private void sendMessage(String approvalType, String selfNo, TaskResp taskResp, Long toUserId) {
        int result = taskResp.getResult();
        String content = null;
        String assigneeUserName = taskResp.getAssigneeUser().getNickname();
        String reason = taskResp.getComment();
        String title = null;
        if (result == BpmProcessInstanceResultEnum.APPROVE.getResult()) {
            content = "您好，您提交的出货单#selfNo#，#approvalType#审批单号：#审批单号#已审批通过，请注意查看";
            title = "#selfNo##approvalType#审核通过";
        } else if (result == BpmProcessInstanceResultEnum.REJECT.getResult()) {
            content = "您好，您提交的出货单#selfNo#，#approvalType#审批单号:#审批单号#审批未通过，由#asasigneeUserName#驳回原因：#reason#，请注意查看";
            title = "#selfNo##approvalType#审核驳回";
        }
        if (StringUtils.isNotEmpty(content)) {
            title = title.replace("#selfNo#", selfNo);
            title = title.replace("#approvalType#", approvalType);

            content = content.replace("#selfNo#", selfNo);
            content = content.replace("#approvalType#", approvalType);
            content = content.replace("#asasigneeUserName#", assigneeUserName);
            content = content.replace("#reason#", reason);

            InternalMessageCreateReqVO createReqVO = new InternalMessageCreateReqVO();
            createReqVO.setContent(content);
            createReqVO.setSendTime(new Date());
            createReqVO.setType(2);
            createReqVO.setTitle(title);
            createReqVO.setFromId("1L");
            createReqVO.setToIdList(Collections.singletonList(toUserId));
            internalMessageService.createInternalMessage(createReqVO);
        }
    }

    @Override
    public BoxLoadDetailVO approvalDetail(Long approvalId) {
        BoxApprovalDO boxApprovalDO = getBoxApproval(approvalId);
        Long shipmentId = boxApprovalDO.getShipmentId();
        BoxBaseBackVO boxBackVO = boxService.getBaseBoxInfo(shipmentId);

        BoxLoadDetailBackVO boxLoadDetailBackVO;
        //查询订单详情
        if (boxApprovalDO.getApprovalType() == BoxApprovalTypeEnum.PRELOAD.getType()) {
            boxLoadDetailBackVO = boxService.boxGoodsList(shipmentId, null, 0, 0);
        } else {
            boxLoadDetailBackVO = boxService.boxGoodsList(shipmentId, null, null, 0);
        }

        List<BoxLoadOrderVO> orderList = boxLoadDetailBackVO.getSectionOrderList();
        if(CollectionUtil.isNotEmpty(orderList)) {
            orderList.sort((o1, o2) -> {
                int sectionIndexCompare = Integer.compare(o1.getSectionIndex(), o2.getSectionIndex());
                if (sectionIndexCompare != 0) {
                    return sectionIndexCompare;
                } else {
                    // 如果sectionIndex相同，则按照tidanNum排序
                    return Integer.compare(o1.getTidanNum(), o2.getTidanNum());
                }
            });
        }

        BoxLoadDetailVO approvalDetailVO = new BoxLoadDetailVO();
        approvalDetailVO.setBoxBackVO(boxBackVO);
        approvalDetailVO.setLoadDetail(boxLoadDetailBackVO);
        approvalDetailVO.setApprovalInfo(BoxApprovalConvert.INSTANCE.convert(boxApprovalDO));

        BoxCabinetDO boxCabinetDO = boxService.getBoxCabinetDO(shipmentId);
        if (boxCabinetDO != null) {
            approvalDetailVO.setCabinetVO(BoxCabinetConvert.INSTANCE.convert(boxCabinetDO));
        }

        BoxCabinetUnloadDO boxCabinetUnloadDO = boxService.getBoxCabinetUnloadDO(shipmentId);
        if (boxCabinetUnloadDO != null) {
            approvalDetailVO.setCabinetUnloadVO(BoxCabinetUnloadConvert.INSTANCE.convert(boxCabinetUnloadDO));
        }
        return approvalDetailVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void cancelApproval(BoxApprovalUpdateReqVO reqVO) {
        Long id = reqVO.getId();
        BoxApprovalDO boxApprovalDO = getBoxApproval(id);
        String bmpId = boxApprovalDO.getBpmProcessId();
        //回调工作流
        bpmCreateServiceFactory.cancelBmpById(SecurityFrameworkUtils.getLoginUserId(), bmpId, reqVO.getApplyReason());
    }

    @Override
    public void checkApproval(Long shipmentId, Integer approvalType) {
        BoxApprovalDO existDO = boxApprovalMapper.selectOne(new LambdaQueryWrapperX<BoxApprovalDO>()
                .eq(BoxApprovalDO::getShipmentId, shipmentId)
                .eq(BoxApprovalDO::getApprovalStatus, BpmProcessInstanceResultEnum.PROCESS.getResult())
                .eqIfPresent(BoxApprovalDO::getApprovalType, approvalType)
                .last("limit 1")
        );
        if (existDO != null) {
            if (approvalType == null) approvalType = existDO.getApprovalType();
            BoxApprovalTypeEnum boxApprovalTypeEnum = BoxApprovalTypeEnum.getApprovalTypeByTypeValue(approvalType);
            Long orderId = existDO.getOrderId();
            String orderNo = null;
            if(orderId != null && orderId > 0) {
                OrderDO orderDO = boxService.getOrder(orderId);
                orderNo = orderDO.getOrderNo();
            }
            throw new ServiceException(500, "当前出货单正在" + boxApprovalTypeEnum.getText() + "审核中" + (StringUtils.isNotBlank(orderNo) ? ",订单号为:" + orderNo : ""));
        }
    }

    @Override
    public Integer getBackApprovalType(Long shipmentId) {
        List<Integer> backTypeList = Arrays.asList(BoxApprovalTypeEnum.PRELOAD_UMPIRE.getType(),
                BoxApprovalTypeEnum.UNLOAD_UMPIRE.getType(), BoxApprovalTypeEnum.SEALING_CABINET_UMPIRE.getType(),
                BoxApprovalTypeEnum.AIR_SORTING_BACK.getType(), BoxApprovalTypeEnum.AIR_TO_WAREHOUSE_BACK.getType(),
                BoxApprovalTypeEnum.AIR_SHIPMENT_BACK.getType()
                );
        BoxApprovalDO existDO = boxApprovalMapper.selectOne(new LambdaQueryWrapperX<BoxApprovalDO>()
                .eq(BoxApprovalDO::getShipmentId, shipmentId)
                .eq(BoxApprovalDO::getApprovalStatus, BpmProcessInstanceResultEnum.PROCESS.getResult())
                .in(BoxApprovalDO::getApprovalType, backTypeList)
                .last("limit 1")
        );
        if (existDO != null) {
            return existDO.getApprovalType();
        }
        return null;
    }

    @Override
   public List<BoxApprovalDO> getBackApprovalList(List<Long> idList) {
       List<Integer> backTypeList = Arrays.asList(BoxApprovalTypeEnum.PRELOAD_UMPIRE.getType(),
               BoxApprovalTypeEnum.UNLOAD_UMPIRE.getType(), BoxApprovalTypeEnum.SEALING_CABINET_UMPIRE.getType(),
               BoxApprovalTypeEnum.AIR_SORTING_BACK.getType(), BoxApprovalTypeEnum.AIR_TO_WAREHOUSE_BACK.getType(),
               BoxApprovalTypeEnum.AIR_SHIPMENT_BACK.getType()
       );
       return boxApprovalMapper.selectList(new LambdaQueryWrapperX<BoxApprovalDO>()
               .in(BoxApprovalDO::getShipmentId, idList)
               .eq(BoxApprovalDO::getApprovalStatus, BpmProcessInstanceResultEnum.PROCESS.getResult())
               .in(BoxApprovalDO::getApprovalType, backTypeList)
       );
   }
}
